﻿Namespace N4N.xml.helper
   ''' <summary>
   ''' Copyright © 2016 Nick4Name di Nicola Martella & F. s.a.s
   '''     This file is part of XMLSmartHelper.
   ''' XMLSmartHelper Is free software: you can redistribute it And/Or modify
   ''' it under the terms Of the GNU Lesser General Public License As published by
   ''' the Free Software Foundation, either version 3 Of the License, Or
   ''' (at your option) any later version.
   ''' 
   ''' XMLSmartHelper Is distributed In the hope that it will be useful,
   ''' but WITHOUT ANY WARRANTY; without even the implied warranty Of
   ''' MERCHANTABILITY Or FITNESS FOR A PARTICULAR PURPOSE.  See the
   ''' GNU Lesser General Public License For more details.
   ''' 
   ''' You should have received a copy Of the GNU Lesser General Public License
   ''' along with XMLSmartHelper.  If Not, see <http://www.gnu.org/licenses/>.
   ''' 
   ''' Email Author: nick4name@gmail.com
   ''' </summary>
   Public Class XMLHelper

#Region " Variabili "

      Private _filename As String

#End Region ' Variabili

#Region " Costruttori "

      ''' <summary>
      ''' Nome file in cui l'XML viene aggiornato a fronte di ogni metodo di scrittura e aggiornamento.
      ''' </summary>
      ''' <param name="filename"></param>
      ''' <remarks></remarks>
      Public Sub New(filename As String)
         _filename = filename
      End Sub

#End Region ' Costruttori

#Region " Metodi "

      ''' <summary>
      ''' Aggiunge all'XML l'elemento root name.
      ''' </summary>
      ''' <param name="name"></param>
      ''' <returns>elID, GUID univoco dell'elemento.</returns>
      ''' <remarks></remarks>
      Public Function createRootElement(name As String) As ElID
         Dim id As String = GetNextGUID()

         Dim x As XDocument = loadXmlFromFile()
         Dim el As XElement = New XElement(name, New XAttribute("elID", id))

         x.AddFirst(el)
         x.Save(_filename)

         Dim elid As ElID = New ElID(name, id)
         Return elid

      End Function

      ''' <summary>
      ''' Aggiunge all'XML l'elemento root name con gli attributi contenuti nella lista di 
      ''' KeyValuePair nome attributo-valore attributo.
      ''' </summary>
      ''' <param name="name"></param>
      ''' <returns>elID, GUID univoco dell'elemento.</returns>
      ''' <remarks></remarks>
      Public Function createRootElement(name As String, attribs As List(Of KeyValuePair(Of String, String))) As ElID
         Dim id As String = GetNextGUID()

         Dim x As XDocument = loadXmlFromFile()
         Dim el As XElement = New XElement(name, New XAttribute("elID", id))
         For i As Integer = 0 To attribs.Count - 1
            el.SetAttributeValue(attribs(i).Key, attribs(i).Value)
         Next

         x.AddFirst(el)
         x.Save(_filename)

         Dim elid As ElID = New ElID(name, id)
         Return elid

      End Function

      ''' <summary>
      ''' Aggiunge all'XML un elemento name dipendente dall'elemento con ElID elIDparent.
      ''' </summary>
      ''' <param name="name"></param>
      ''' <returns>elID, GUID univoco dell'elemento.</returns>
      ''' <remarks></remarks>
      Public Function createChildElement(elIDparent As String, name As String) As ElID
         Dim id As String = GetNextGUID()

         Dim x As XDocument = loadXmlFromFile()

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                                    From elem In x.Elements().DescendantsAndSelf
                                                    Where elem.@elID = elIDparent
                                                    Select elem
         If elems.Count = 0 Then
            Return Nothing
         End If

         Dim el As XElement = New XElement(name, New XAttribute("elID", id))

         elems(0).Add(el)
         x.Save(_filename)

         Dim elid As ElID = New ElID(name, id)
         Return elid

      End Function

      ''' <summary>
      ''' Aggiunge all'XML un elemento name con gli attributi contenuti nella lista di KeyValuePair nome-valore.
      ''' Il nuovo elemento dipende dall'elemento con ElID elIDparent.
      ''' </summary>
      ''' <param name="name"></param>
      ''' <returns>elID, GUID univoco dell'elemento.</returns>
      ''' <remarks></remarks>
      Public Function createChildElement(elIDparent As String, name As String, attribs As List(Of KeyValuePair(Of String, String))) As ElID
         Dim id As String = GetNextGUID()

         Dim x As XDocument = loadXmlFromFile()

         Dim el As XElement = New XElement(name, New XAttribute("elID", id))
         For i As Integer = 0 To attribs.Count - 1
            el.SetAttributeValue(attribs(i).Key, attribs(i).Value)
         Next

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                                    From elem In x.Elements().DescendantsAndSelf
                                                    Where elem.@elID = elIDparent
                                                    Select elem
         If elems.Count = 0 Then
            Return Nothing
         End If

         elems(0).Add(el)
         x.Save(_filename)

         Dim elid As ElID = New ElID(name, id)
         Return elid

      End Function

      ''' <summary>
      ''' Valorizza la proprietà text dell'elemento con ElID elID.
      ''' </summary>
      ''' <param name="elID"></param>
      ''' <param name="value"></param>
      ''' <remarks></remarks>
      Public Sub setTextValue(elID As String, value As String)
         Dim x As XDocument = loadXmlFromFile()

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                              From elem In x.Elements().DescendantsAndSelf
                                              Where elem.@elID = elID
                                              Select elem
         If elems.Count = 0 Then
            Return
         End If

         elems(0).SetValue(value)
         x.Save(_filename)

      End Sub

      ''' <summary>
      ''' Assegna il valore value all'attributo attrib appartenente all'elemento con ElID elID.
      ''' </summary>
      ''' <param name="elID"></param>
      ''' <param name="value"></param>
      ''' <remarks></remarks>
      Public Sub setAttributeValue(elID As String, value As String, attrib As String)
         Dim x As XDocument = loadXmlFromFile()

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                              From elem In x.Elements().DescendantsAndSelf
                                              Where elem.@elID = elID
                                              Select elem
         If elems.Count = 0 Then
            Return
         End If

         elems(0).SetAttributeValue(attrib, value)
         x.Save(_filename)

      End Sub

      ''' <summary>
      ''' Ritorna l'elemento XML e suoi figli a partire all'elemento con ElID elID.
      ''' </summary>
      ''' <param name="elID"></param>
      ''' <returns></returns>
      ''' <remarks></remarks>
      Public Function getElement(elID As String) As XElement
         Dim x As XDocument = loadXmlFromFile()

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                                          From elem In x.Elements().DescendantsAndSelf
                                                          Where elem.@elID = elID
                                                          Select elem
         If elems.Count = 0 Then
            Return Nothing
         End If

         Return elems(0)

      End Function

      ''' <summary>
      ''' Ritorna il nome dell'elemento XML con ElID elID.
      ''' </summary>
      ''' <param name="elID"></param>
      ''' <returns></returns>
      ''' <remarks></remarks>
      Public Function getElementName(elID As String) As ElID
         Dim x As XDocument = loadXmlFromFile()

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                                          From elem In x.Elements().DescendantsAndSelf
                                                          Where elem.@elID = elID
                                                          Select elem
         If elems.Count = 0 Then
            Return Nothing
         End If

         Dim _elid As ElID = New ElID(elems(0).Name.ToString, elID)
         Return _elid

      End Function

      ''' <summary>
      ''' Ritorna il valore dell'attributo attrib appartenente all'elemento con ElID elID.
      ''' </summary>
      ''' <param name="elID"></param>
      ''' <param name="attrib"></param>
      ''' <returns>
      ''' Il valore dell'attributo attrib.
      ''' Se attrib esiste ma è vuoto ritorna empty string;
      ''' se attrib non esiste ritorna nothing.
      ''' </returns>
      ''' <remarks></remarks>
      Public Function getAttributeValue(elID As String, attrib As String) As String
         Dim x As XDocument = loadXmlFromFile()

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                                          From elem In x.Elements().DescendantsAndSelf
                                                          Where elem.@elID = elID
                                                          Select elem
         If elems.Count = 0 Then
            Return Nothing
         End If

         Dim v As String = Nothing
         For Each a As XAttribute In elems.Attributes
            If a.Name = attrib Then
               v = a.Value
            End If
         Next

         Return v

      End Function

      ''' <summary>
      ''' Cancella l'elemento con ElID elID.
      ''' </summary>
      ''' <param name="elID"></param>
      ''' <remarks></remarks>
      Public Sub deleteElID(elID As String)
         Dim x As XDocument = loadXmlFromFile()

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                              From elem In x.Elements().DescendantsAndSelf
                                              Where elem.@elID = elID
                                              Select elem
         If elems.Count = 0 Then
            Return
         End If

         elems(0).Remove()
         x.Save(_filename)

      End Sub

      ''' <summary>
      ''' Rimuove l'attributo attrib appartenente all'elemento con ElID elID.
      ''' </summary>
      ''' <param name="elID"></param>
      ''' <param name="attrib"></param>
      ''' <remarks></remarks>
      Public Sub deleteAttribute(elID As String, attrib As String)
         Dim x As XDocument = loadXmlFromFile()

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                                          From elem In x.Elements().DescendantsAndSelf
                                                          Where elem.@elID = elID
                                                          Select elem
         If elems.Count = 0 Then
            Return
         End If

         Dim v As String = Nothing
         For Each a As XAttribute In elems.Attributes
            If a.Name = attrib Then
               a.Remove()
            End If
         Next

         x.Save(_filename)

      End Sub

      ''' <summary>
      ''' Ritorna una collection List di KeyValuePair degli ElID degli elementi name.
      ''' L'istanza KeyValuePair ha l'ElID in key e l'istanza ElID dell'elemento in value.
      ''' </summary>
      ''' <param name="name"></param>
      ''' <returns></returns>
      ''' <remarks></remarks>
      Public Function getElID(name As String) As List(Of KeyValuePair(Of String, ElID))
         Dim x As XDocument = loadXmlFromFile()
         Dim matched As List(Of KeyValuePair(Of String, ElID)) = retElIDByName(name, x.Root)

         Return matched

      End Function

      ''' <summary>
      ''' Ritorna una collection List di KeyValuePair degli ElID degli elementi name contenuti nell'istanza XElement source.
      ''' L'istanza KeyValuePair ha l'ElID in key e l'istanza ElID dell'elemento in value.
      ''' </summary>
      ''' <param name="name"></param>
      ''' <param name="source"></param>
      ''' <returns></returns>
      ''' <remarks></remarks>
      Public Function getElID(name As String, source As XElement) As List(Of KeyValuePair(Of String, ElID))
         Dim matched As List(Of KeyValuePair(Of String, ElID)) = retElIDByName(name, source)

         Return matched

      End Function

      ''' <summary>
      ''' Ritorna una collection List di KeyValuePair degli ElID degli elementi name aventi gli attributi elencati in attribs.
      ''' L'istanza KeyValuePair ha l'ElID in key e l'istanza ElID dell'elemento in value.
      ''' </summary>
      ''' <param name="name"></param>
      ''' <param name="attribs"></param>
      ''' <returns></returns>
      ''' <remarks></remarks>
      Public Function getElID(name As String, attribs As List(Of KeyValuePair(Of String, String))) _
                                                                     As List(Of KeyValuePair(Of String, ElID))
         Dim x As XDocument = loadXmlFromFile()
         Dim matched As List(Of KeyValuePair(Of String, ElID)) = retElIDByName(name, attribs, x.Root)

         Return matched

      End Function

      ''' <summary>
      ''' Ritorna una collection List di KeyValuePair degli ElID degli elementi name aventi gli attributi elencati in attribs.
      ''' L'istanza KeyValuePair ha l'ElID in key e l'istanza ElID dell'elemento in value.
      ''' </summary>
      ''' <param name="name"></param>
      ''' <param name="attribs"></param>
      ''' <returns></returns>
      ''' <remarks></remarks>
      Public Function getElID(name As String, attribs As List(Of KeyValuePair(Of String, String)), source As XElement) _
                                                                     As List(Of KeyValuePair(Of String, ElID))

         Dim matched As List(Of KeyValuePair(Of String, ElID)) = retElIDByName(name, attribs, source)

         Return matched

      End Function

#End Region ' Metodi

#Region " Funzioni "

      Private Function retElIDByName(name As String, attribs As List(Of KeyValuePair(Of String, String)), source As XElement)
         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                                          From elem In source.Elements().DescendantsAndSelf
                                                          Where elem.Name = name
                                                          Select elem
         If elems.Count = 0 Then
            Return Nothing
         End If

         Dim matched As List(Of KeyValuePair(Of String, ElID)) = New List(Of KeyValuePair(Of String, ElID))
         For Each xe As XElement In elems
            Dim mch As Boolean = False
            Dim mchOk As Boolean = False
            For i As Integer = 0 To attribs.Count - 1
               For Each xa As XAttribute In xe.Attributes
                  If (xa.Name = attribs(i).Key) And (xa.Value = attribs(i).Value) Then
                     mch = True
                     Exit For
                  End If
               Next
               If mch Then
                  mchOk = True
                  mch = False
               Else
                  mchOk = False
                  Exit For
               End If
            Next
            If mchOk Then
               Dim elid As ElID = New ElID(xe.Name.ToString, xe.@elID)
               Dim m As KeyValuePair(Of String, ElID) =
                  New KeyValuePair(Of String, ElID)(xe.@elID, elid)
               mch = False
               matched.Add(m)
            End If
         Next

         Return matched
      End Function

      Private Function retElIDByName(name As String, source As XElement)

         Dim elems As System.Collections.Generic.IEnumerable(Of XElement) =
                                                          From elem In source.DescendantsAndSelf
                                                          Where elem.Name = name
                                                          Select elem
         If elems.Count = 0 Then
            Return Nothing
         End If

         Dim matched As List(Of KeyValuePair(Of String, ElID)) = New List(Of KeyValuePair(Of String, ElID))
         For Each e As XElement In elems
            Dim elid As ElID = New ElID(e.Name.ToString, e.@elID)
            Dim kv As KeyValuePair(Of String, ElID) = New KeyValuePair(Of String, ElID)(e.@elID, elid)
            matched.Add(kv)
         Next

         Return matched

      End Function

      Private Function GetNextGUID() As String
         Dim guid As System.Guid = New System.Guid("00000000000000000000000000000000")
         Return Guid.NewGuid.ToString()
      End Function

      Private Function loadXmlFromFile() As XDocument
         Dim x As XDocument = New XDocument()

         If System.IO.File.Exists(_filename) Then
            x = XDocument.Load(_filename)
         End If

         Return x

      End Function

#End Region ' Funzioni 

   End Class
End Namespace